/*
   Project name:
     Arduino Ground Balance Test - Subtractive Method
   Copyright:
     (c) George Overton, 2021.
   Revision History:
     - V1.0 Initial release
   Description:
     GB is based on a subtractive technique
     Target delay (TGD), target width (TGW), and GB delay (GBD) are fixed
     GB width (GBW) is variable by user
     Target EF width equals TGW
     GB EF width equals GBW
     EF delays are fixed
     Delay between EF samples equals GBD
   NOTES:
     TX OSC signal must be present for +5V supply to work.
*/

// Pin assignments
// D0 and D1 are used by hardware UART
byte txPin = 8;                         // Assign TX output to D8
byte s1Pin = 9;                         // Assign sample 1 pulse to D9
byte s2Pin = 10;                        // Assign sample 2 pulse to D10
byte audioPin = 11;                     // Assign audio chopper to D11
byte gbSwPin = 12;                      // Assign GB switch to D12
// D13 is attached to an onboard LED
// Use only as an output
// Requires external pull-down resistor if used as an input
byte potPin = A0;                       // Assign pot to A0
// A6 and A7 can only be used as inputs

// Program constants
// Initial detector timings are in micro-seconds
const unsigned int initTx = 100;          // TX-on pulse
const unsigned int initTxPeriod = 1000;   // TX period
const unsigned int initTgd = 17;          // Initial target delay relative to TX end
const unsigned int initTgwOff = 50;       // Initial target width (GB off) relative to TGD end
const unsigned int initTgwOn = 10;        // Initial target width (GB on) relative to TGD end
const unsigned int initGbd = 10;          // Initial GB delay
const unsigned int initGbw = 10;          // Initial GB width
const unsigned int initEfd = 500;         // Initial EF delay relative to TX end
const byte readPotLimit = 100;            // Wait 10 TX periods before reading pot
const byte mosfetOn = LOW;                // Mosfet turns on when transmitter input high
const byte mosfetOff = HIGH;              // Mosfet turns off when transmitter input low
const byte syncDemodOn = LOW;             // Sample gate on pulse
const byte syncDemodOff = HIGH;           // Sample gate off pulse
const byte gbOff = HIGH;                  // GB off when switch input high
const byte gbOn = LOW;                    // GB on when switch input low

// Detector timings
unsigned int tx = initTx;              // TX pulse
unsigned int txPeriod = initTxPeriod;  // TX period
unsigned int tgd = initTgd;            // Target sample delay
unsigned int tgwOff = initTgwOff;      // Target sample width (GB off)
unsigned int tgwOn = initTgwOn;        // Target sample width (GB on)
unsigned int gbd = initGbd;            // GB sample delay
unsigned int gbw = initGbw;            // GB sample width
unsigned int efd = initEfd;            // EF sample delay

// Timing offsets for GB off
unsigned int txOffsetOff = 4;         // TX pulse offset
unsigned int txPeriodOffsetOff = 10;  // TX period offset
unsigned int tgdOffsetOff = 5;        // TGD offset
unsigned int tgwOffsetOff = 6;        // TGW offset
unsigned int efdOffsetOff = 12;       // EFD offset
unsigned int efwOffsetOff = 1;        // EFW offset

// Timing offsets for GB on
unsigned int txOffsetOn = 4;          // TX pulse offset
unsigned int txPeriodOffsetOn = 120;  // TX period offset
unsigned int tgdOffsetOn = 5;         // TGD offset
unsigned int tgwOffsetOn = 6;         // TGW offset
unsigned int gbdOffset = 5;           // GBD offset
unsigned int gbwOffset = 7;           // GBW offset
unsigned int efdOffsetTg = 8;         // EFD offset for target pulse
unsigned int efwOffsetTg = 7;         // EFW offset for target pulse
unsigned int efdOffsetGb = 8;         // EFD offset for GB pulse
unsigned int efwOffsetGb = 6;         // EFW offset for GB pulse

// Program variables
unsigned int txCount;                    // TX pulse
unsigned int txPeriodCount;              // TX period
unsigned int tgdCount;                   // Target delay count
unsigned int tgwCount;                   // Target width count
unsigned int gbdCount;                   // GB delay count
unsigned int gbwCount;                   // GB width count
unsigned int efdCountTg;                 // EF delay count for target pulse
unsigned int efwCountTg;                 // EF width count for target pulse
unsigned int efdCountGb;                 // EF delay count for GB pulse
unsigned int efwCountGb;                 // EF width count for GB pulse
word potVal = 0;                         // Pot value
byte gbSw;                               // GB switch position (on or off)
byte gbState;                            // Current state of GB (on or off)

void setup() {
  pinMode(txPin, OUTPUT);             // Set TX pin to output mode
  pinMode(s1Pin, OUTPUT);             // Set S1 pin to output mode
  pinMode(s2Pin, OUTPUT);             // Set S2 pin to output mode
  pinMode(gbSwPin, INPUT_PULLUP);     // Set GB switch pin to input mode with pullup resistor
  gbState = gbOff;                    // Set GB switch state to off
  calcTimerValuesGbOff();             // Calculate timer values when GB is off
  noInterrupts();                     // Disable interrupts
  digitalWrite(txPin, mosfetOff);     // Turn off Mosfet
  digitalWrite(s1Pin, syncDemodOff);  // Turn off S1 gate
  digitalWrite(s2Pin, syncDemodOff);  // Turn off S2 gate
  analogWrite(audioPin, 127);         // Set audioPin with 50% duty cycle PWM
}

// Calculate timer values when GB is off
void calcTimerValuesGbOff() {
  txCount = tx - txOffsetOff; 
  tgdCount = tgd - tgdOffsetOff;
  tgwCount = tgwOff - tgwOffsetOff;
  efdCountTg = efd - tgd - tgwOff - efdOffsetTg;
  efwCountTg = tgwOff - efwOffsetTg;
  txPeriodCount = txPeriod - tx - efd - tgwOff - txPeriodOffsetOff;
}

// Calculate timer values when GB is on
void calcTimerValuesGbOn() {
  txCount = tx - txOffsetOn; 
  tgdCount = tgd - tgdOffsetOn;
  tgwCount = tgwOn - tgwOffsetOn;
  gbdCount = gbd - gbdOffset;
  gbwCount = gbw - gbwOffset;
  efdCountTg = efd - tgd - tgwOn - gbd - gbw - efdOffsetTg;
  efwCountTg = tgwOn - efwOffsetTg;
  efdCountGb = gbd - efdOffsetGb;
  efwCountGb = gbw - efwOffsetGb;
  txPeriodCount = txPeriod - tx - efd - tgwOn - gbd - gbw - txPeriodOffsetOn;
}

void loop() {
  if (gbState == gbOff) {
    
    digitalWrite(txPin, mosfetOn);      // Turn on mosfet
    delayMicroseconds(txCount);         // TX pulse width
    
    digitalWrite(txPin, mosfetOff);     // Turn off mosfet
    delayMicroseconds(tgdCount);        // Target sample delay
    
    digitalWrite(s1Pin, syncDemodOn);   // Turn on sample gate 1
    delayMicroseconds(tgwCount);        // Target sample width
    
    digitalWrite(s1Pin, syncDemodOff);  // Turn off sample gate 1
    delayMicroseconds(efdCountTg);      // EF sampledelay
    
    digitalWrite(s2Pin, syncDemodOn);   // Turn on sample gate 2
    delayMicroseconds(efwCountTg);      // EF sample width
    
    digitalWrite(s2Pin, syncDemodOff);  // Turn off sample gate 2
    delayMicroseconds(txPeriodCount);   // TX period delay
    
  } else {
    
    digitalWrite(txPin, mosfetOn);      // Turn on mosfet
    delayMicroseconds(txCount);         // TX pulse width
    
    digitalWrite(txPin, mosfetOff);     // Turn off mosfet
    delayMicroseconds(tgdCount);        // Target sample delay
    
    digitalWrite(s1Pin, syncDemodOn);   // Turn on sample gate 1
    delayMicroseconds(tgwCount);        // Target sample width
    
    digitalWrite(s1Pin, syncDemodOff);  // Turn off sample gate 1
    delayMicroseconds(gbdCount);        // GB sample delay
    
    digitalWrite(s2Pin, syncDemodOn);   // Turn on sample gate 2
    delayMicroseconds(gbwCount);        // GB sample width
    
    digitalWrite(s2Pin, syncDemodOff);  // Turn off sample gate 2
    delayMicroseconds(efdCountTg);      // EF sample delay (target)
    
    digitalWrite(s2Pin, syncDemodOn);   // Turn on sample gate 2 
    delayMicroseconds(efwCountTg);      // EF sample width (target)
    
    digitalWrite(s2Pin, syncDemodOff);  // Turn off sample gate 2
    delayMicroseconds(efdCountGb);      // EF sample delay (GB)
    
    digitalWrite(s1Pin, syncDemodOn);   // Turn on sample gate 1
    delayMicroseconds(efwCountGb);      // EF sample width (GB)
    
    digitalWrite(s1Pin, syncDemodOff);  // Turn off sample gate 1
    delayMicroseconds(txPeriodCount);   // TX period delay
    
    potVal = analogRead(potPin);        // Read pot value
    gbw = initGbw + potVal / 16;        // Adjust GB sample width
    calcTimerValuesGbOn();              // Calculate new timer values for GB on
  }

  gbSw = digitalRead(gbSwPin);          // Get GB switch position
  if (gbState != gbSw) {                // Check if current GB state has changed
    if (gbSw == gbOff) {                // Check if state changed
      calcTimerValuesGbOff();
    } else {
      calcTimerValuesGbOn();
    }
    gbState = gbSw;                     // Save GB switch state
  }
}
